@@ -22,7 +22,7 @@ from .numpy_compat import ma_divide, divide as np_divide
 from ..compatibility import getargspec, builtins
 from ..base import tokenize
 from ..highlevelgraph import HighLevelGraph
-from ..utils import ignoring, funcname, Dispatch
+from ..utils import ignoring, funcname, Dispatch, deepmap
 from .. import config
 
 # Generic functions to support chunks of different types
@@ -384,57 +384,45 @@ def moment_chunk(A, order=2, sum=chunk.sum, numel=numel, dtype='f8', **kwargs):
     total = sum(A, dtype=dtype, **kwargs)
     n = numel(A, **kwargs).astype(np.int64)
     u = total / n
-    empty = empty_lookup.dispatch(type(n))
-    M = empty(n.shape + (order - 1,), dtype=dtype)
-    for i in range(2, order + 1):
-        M[..., i - 2] = sum((A - u)**i, dtype=dtype, **kwargs)
-    result = empty(n.shape, dtype=[('total', total.dtype),
-                                   ('n', n.dtype),
-                                   ('M', M.dtype, (order - 1,))])
-    result['total'] = total
-    result['n'] = n
-    result['M'] = M
-    return result
+    xs = [sum((A - u)**i, dtype=dtype, **kwargs) for i in range(2, order + 1)]
+    M = np.stack(xs, axis=-1)
+    return {'total': total, 'n': n, 'M': M}
 
 
-def _moment_helper(Ms, ns, inner_term, order, sum, kwargs):
-    M = Ms[..., order - 2].sum(**kwargs) + sum(ns * inner_term ** order, **kwargs)
+def _moment_helper(Ms, ns, inner_term, order, sum, axis, kwargs):
+    M = Ms[..., order - 2].sum(axis=axis, **kwargs) + sum(ns * inner_term ** order, axis=axis, **kwargs)
     for k in range(1, order - 1):
         coeff = factorial(order) / (factorial(k) * factorial(order - k))
-        M += coeff * sum(Ms[..., order - k - 2] * inner_term**k, **kwargs)
+        M += coeff * sum(Ms[..., order - k - 2] * inner_term**k, axis=axis, **kwargs)
     return M
 
 
-def moment_combine(data, order=2, ddof=0, dtype='f8', sum=np.sum, **kwargs):
+def moment_combine(pairs, order=2, ddof=0, dtype='f8', sum=np.sum, axis=None, **kwargs):
+    if not isinstance(pairs, list):
+        pairs = [pairs]
+    totals = _concatenate2(deepmap(lambda pair: pair['total'], pairs), axes=axis)
+    ns = _concatenate2(deepmap(lambda pair: pair['n'], pairs), axes=axis)
+    Ms = _concatenate2(deepmap(lambda pair: pair['M'], pairs), axes=axis)
+
     kwargs['dtype'] = dtype
     kwargs['keepdims'] = True
 
-    totals = data['total']
-    ns = data['n']
-    Ms = data['M']
-    total = totals.sum(**kwargs)
-    n = sum(ns, **kwargs)
+    total = totals.sum(axis=axis, **kwargs)
+    n = ns.sum(axis=axis, **kwargs)
     mu = divide(total, n, dtype=dtype)
     inner_term = divide(totals, ns, dtype=dtype) - mu
-    empty = empty_lookup.dispatch(type(n))
-    M = empty(n.shape + (order - 1,), dtype=dtype)
 
-    for o in range(2, order + 1):
-        M[..., o - 2] = _moment_helper(Ms, ns, inner_term, o, sum, kwargs)
-
-    result = empty(n.shape, dtype=[('total', total.dtype),
-                                   ('n', n.dtype),
-                                   ('M', Ms.dtype, (order - 1,))])
-    result['total'] = total
-    result['n'] = n
-    result['M'] = M
-    return result
+    xs = [_moment_helper(Ms, ns, inner_term, o, sum, axis, kwargs) for o in range(2, order + 1)]
+    M = np.stack(xs, axis=-1)
+    return {'total': total, 'n': n, 'M': M}
 
 
-def moment_agg(data, order=2, ddof=0, dtype='f8', sum=np.sum, **kwargs):
-    totals = data['total']
-    ns = data['n']
-    Ms = data['M']
+def moment_agg(pairs, order=2, ddof=0, dtype='f8', sum=np.sum, axis=None, **kwargs):
+    if not isinstance(pairs, list):
+        pairs = [pairs]
+    totals = _concatenate2(deepmap(lambda pair: pair['total'], pairs), axes=axis)
+    ns = _concatenate2(deepmap(lambda pair: pair['n'], pairs), axes=axis)
+    Ms = _concatenate2(deepmap(lambda pair: pair['M'], pairs), axes=axis)
 
     kwargs['dtype'] = dtype
     # To properly handle ndarrays, the original dimensions need to be kept for
@@ -442,12 +430,12 @@ def moment_agg(data, order=2, ddof=0, dtype='f8', sum=np.sum, **kwargs):
     keepdim_kw = kwargs.copy()
     keepdim_kw['keepdims'] = True
 
-    n = sum(ns, **keepdim_kw)
-    mu = divide(totals.sum(**keepdim_kw), n, dtype=dtype)
+    n = ns.sum(axis=axis, **keepdim_kw)
+    mu = divide(totals.sum(axis=axis, **keepdim_kw), n, dtype=dtype)
     inner_term = divide(totals, ns, dtype=dtype) - mu
 
-    M = _moment_helper(Ms, ns, inner_term, order, sum, kwargs)
-    return divide(M, sum(n, **kwargs) - ddof, dtype=dtype)
+    M = _moment_helper(Ms, ns, inner_term, order, sum, axis, kwargs)
+    return divide(M, n.sum(axis=axis, **kwargs) - ddof, dtype=dtype)
 
 
 def moment(a, order, axis=None, dtype=None, keepdims=False, ddof=0,
@@ -471,6 +459,7 @@ def moment(a, order, axis=None, dtype=None, keepdims=False, ddof=0,
                      partial(moment_agg, order=order, ddof=ddof),
                      axis=axis, keepdims=keepdims,
                      dtype=dt, split_every=split_every, out=out,
+                     concatenate=False,
                      combine=partial(moment_combine, order=order))
 
 
@@ -483,7 +472,8 @@ def var(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None,
         dt = getattr(np.var(np.ones(shape=(1,), dtype=a.dtype)), 'dtype', object)
     return reduction(a, moment_chunk, partial(moment_agg, ddof=ddof), axis=axis,
                      keepdims=keepdims, dtype=dt, split_every=split_every,
-                     combine=moment_combine, name='var', out=out)
+                     combine=moment_combine, name='var', out=out,
+                     concatenate=False)
 
 
 def nanvar(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None,
@@ -495,7 +485,8 @@ def nanvar(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None,
     return reduction(a, partial(moment_chunk, sum=chunk.nansum, numel=nannumel),
                      partial(moment_agg, sum=np.nansum, ddof=ddof), axis=axis,
                      keepdims=keepdims, dtype=dt, split_every=split_every,
-                     combine=partial(moment_combine, sum=np.nansum), out=out)
+                     combine=partial(moment_combine, sum=np.nansum), out=out,
+                     concatenate=False)
 
 
 with ignoring(AttributeError):
