@@ -16,10 +16,10 @@ from .. import config
 from ..compatibility import unicode
 from ..base import tokenize
 from ..delayed import delayed
-from ..utils import import_required, is_integer
+from ..utils import import_required, is_integer, parse_bytes
 
 
-def read_bytes(urlpath, delimiter=None, not_zero=False, blocksize=2**27,
+def read_bytes(urlpath, delimiter=None, not_zero=False, blocksize="128 MiB",
                sample=True, compression=None, include_path=False, **kwargs):
     """Given a path or paths, return delayed objects that read from those paths.
 
@@ -44,8 +44,8 @@ def read_bytes(urlpath, delimiter=None, not_zero=False, blocksize=2**27,
         bytes.
     not_zero : bool
         Force seek of start-of-file delimiter, discarding header.
-    blocksize : int (=128MB)
-        Chunk size in bytes
+    blocksize : int, str
+        Chunk size in bytes, defaults to "128 MiB"
     compression : string or None
         String like 'gzip' or 'xz'.  Must support efficient random access.
     sample : bool or int
@@ -83,6 +83,8 @@ def read_bytes(urlpath, delimiter=None, not_zero=False, blocksize=2**27,
         raise IOError("%s resolved to no files" % urlpath)
 
     if blocksize is not None:
+        if isinstance(blocksize, (str, unicode)):
+            blocksize = parse_bytes(blocksize)
         if not is_integer(blocksize):
             raise TypeError("blocksize must be an integer")
         blocksize = int(blocksize)
@@ -295,6 +297,7 @@ def expand_paths_if_needed(paths, mode, num, fs, name_function):
         raise ValueError("When writing data, only one filename mask can be specified.")
     for curr_path in paths:
         if '*' in curr_path:
+            glob = True
             if 'w' in mode:
                 # expand using name_function
                 expanded_paths.extend(_expand_paths(curr_path, name_function, num))
@@ -302,9 +305,10 @@ def expand_paths_if_needed(paths, mode, num, fs, name_function):
                 # expand using glob
                 expanded_paths.extend(fs.glob(curr_path))
         else:
+            glob = False
             expanded_paths.append(curr_path)
     # if we generated more paths that asked for, trim the list
-    if 'w' in mode and len(expanded_paths) > num:
+    if 'w' in mode and len(expanded_paths) > num and glob:
         expanded_paths = expanded_paths[:num]
     return expanded_paths
 
