@@ -243,6 +243,22 @@ def test_getitem_slice():
     assert_eq(ddf['f':], df['f':])
 
 
+def test_getitem_integer_slice():
+    df = pd.DataFrame({'A': range(6)})
+    ddf = dd.from_pandas(df, 2)
+    # integer slicing is iloc based
+    with pytest.raises(NotImplementedError):
+        ddf[1:3]
+
+    df = pd.DataFrame({'A': range(6)},
+                      index=[1., 2., 3., 5., 10., 11.])
+    ddf = dd.from_pandas(df, 2)
+    # except for float dtype indexes
+    assert_eq(ddf[2:8], df[2:8])
+    assert_eq(ddf[2:], df[2:])
+    assert_eq(ddf[:8], df[:8])
+
+
 def test_loc_on_numpy_datetimes():
     df = pd.DataFrame({'x': [1, 2, 3]},
                       index=list(map(np.datetime64, ['2014', '2015', '2016'])))
@@ -392,3 +408,44 @@ def test_to_series():
     ddf = dd.from_pandas(df, 10)
 
     assert_eq(df.index.to_series(), ddf.index.to_series())
+
+
+@pytest.mark.parametrize('indexer', [
+    0,
+    [0],
+    [0, 1],
+    [1, 0],
+    [False, True, True]
+])
+def test_iloc(indexer):
+    df = pd.DataFrame({"A": [1, 2], "B": [3, 4], "C": [5, 6]})
+    ddf = dd.from_pandas(df, 2)
+
+    result = ddf.iloc[:, indexer]
+    expected = df.iloc[:, indexer]
+
+    assert_eq(result, expected)
+
+
+def test_iloc_series():
+    s = pd.Series([1, 2, 3])
+    ds = dd.from_pandas(s, 2)
+    with pytest.raises(AttributeError):
+        ds.iloc[:]
+
+
+def test_iloc_raises():
+    df = pd.DataFrame({"A": [1, 2], "B": [3, 4], "C": [5, 6]})
+    ddf = dd.from_pandas(df, 2)
+
+    with pytest.raises(NotImplementedError):
+        ddf.iloc[[0, 1], :]
+
+    with pytest.raises(NotImplementedError):
+        ddf.iloc[[0, 1], [0, 1]]
+
+    with pytest.raises(ValueError):
+        ddf.iloc[[0, 1], [0, 1], [1, 2]]
+
+    with pytest.raises(IndexError):
+        ddf.iloc[:, [5, 6]]
